==> view_mode_switch-8.x-1.2.tar.gz: view_mode_switch/src/Plugin/Field/FieldWidget/ViewModeSwitchWidget.php <==
<?php

namespace Drupal\view_mode_switch\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'view_mode_switch' widget.
 *
 * @FieldWidget(
 *   id = "view_mode_switch",
 *   label = @Translation("View mode switch"),
 *   field_types = {
 *     "view_mode_switch",
 *   },
 * )
 */
class ViewModeSwitchWidget extends WidgetBase implements ContainerFactoryPluginInterface {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * Constructs a ViewModeSwitchWidget object.
   *
   * @param string $plugin_id
   *   The plugin_id for the widget.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the widget is associated.
   * @param array $settings
   *   The widget settings.
   * @param array $third_party_settings
   *   Any third party settings.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, array $third_party_settings, AccountInterface $current_user) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);

    $this->currentUser = $current_user;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['third_party_settings'],
      $container->get('current_user')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state): array {
    $value = isset($items[$delta]->value) ? $items[$delta]->value : '';
    // Determine view modes allowed to switch to.
    $options = $this->fieldDefinition
      ->getFieldStorageDefinition()
      ->getOptionsProvider('value', $items->getEntity())
      ->getSettableOptions($this->currentUser);

    $is_required = $this->fieldDefinition->isRequired();

    if (!$is_required || $items[$delta]->isEmpty() || !isset($options[$value])) {
      $options = [
        '' => !$is_required ? $this->t('- No change -') : $this->t('- Select -'),
      ] + $options;
    }

    // View mode to switch to (if set).
    $element['value'] = $element + [
      '#type' => 'select',
      '#options' => $options,
      '#default_value' => $value,
    ];

    return $element;
  }

}

==> view_mode_switch-8.x-1.3.tar.gz: view_mode_switch/src/Plugin/Field/FieldWidget/ViewModeSwitchWidget.php <==
<?php

namespace Drupal\view_mode_switch\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Field\WidgetInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\TypedData\OptionsProviderInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'view_mode_switch' widget.
 *
 * @FieldWidget(
 *   id = "view_mode_switch",
 *   label = @Translation("View mode switch"),
 *   field_types = {
 *     "view_mode_switch",
 *   },
 * )
 */
class ViewModeSwitchWidget extends WidgetBase {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);

    // Inject current user.
    $instance->setCurrentUser($container->get('current_user'));

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state): array {
    $value = isset($items[$delta]->value) ? $items[$delta]->value : '';

    // Determine view modes allowed to switch to.
    $options_provider = $this->fieldDefinition
      ->getFieldStorageDefinition()
      ->getOptionsProvider('value', $items->getEntity());
    $options = $options_provider instanceof OptionsProviderInterface ? $options_provider->getSettableOptions($this->currentUser) : [];

    $is_required = $this->fieldDefinition->isRequired();

    if (!$is_required || $items[$delta]->isEmpty() || !isset($options[$value])) {
      $options = [
        '' => !$is_required ? $this->t('- No change -') : $this->t('- Select -'),
      ] + $options;
    }

    // View mode to switch to (if set).
    $element['value'] = $element + [
      '#type' => 'select',
      '#options' => $options,
      '#default_value' => $value,
    ];

    return $element;
  }

  /**
   * Sets the current user.
   *
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   *
   * @return static
   *   The object itself for chaining.
   */
  public function setCurrentUser(AccountInterface $current_user): WidgetInterface {
    $this->currentUser = $current_user;

    return $this;
  }

}
